#ifndef CHIPLET_INTERFACE_H
#define CHIPLET_INTERFACE_H

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_target_socket.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <vector>
#include <src/utils/configuration.h>
#include <src/utils/noc_logger.h>

using namespace tlm;

class O_E_converter; // Forward declaration

class ChipletInterface : public sc_module {
public:
    // TLM sockets
    tlm_utils::simple_target_socket<ChipletInterface> interposer_features_receive_socket; // Receiving from Interposer Interface
    tlm_utils::simple_target_socket<ChipletInterface> interposer_weight_receive_socket; // Receiving from Interposer Interface
    tlm_utils::simple_initiator_socket<ChipletInterface> interposer_send_socket;   // Sending to Interposer Interface    
    unsigned int chiplet_id;

    // Sockets for PEs
    tlm_utils::simple_target_socket<ChipletInterface> oec_receive_sockets[PE_PER_CHIPLET]; // Receiving from O_E_converters
    tlm_utils::simple_initiator_socket<ChipletInterface> oec_features_send_sockets[PE_PER_CHIPLET];   // Sending to O_E_converters
    tlm_utils::simple_initiator_socket<ChipletInterface> oec_weight_send_sockets[PE_PER_CHIPLET];   // Sending to O_E_converters

    SC_HAS_PROCESS(ChipletInterface);

    explicit ChipletInterface(sc_module_name name, int num_pes);

private:
    int num_pes;  // Number of PEs

    // Handle incoming transactions from the Interposer Interface
    void ii_fetures_receive_b_transport(tlm_generic_payload& trans, sc_time& delay);
    void ii_weight_receive_b_transport(tlm_generic_payload& trans, sc_time& delay);
    
    // Handle incoming transactions from the OEC
    void oec_b_transport(tlm_generic_payload& trans, sc_time& delay);
    // Forward transaction to O_E_converters
    void forward_features_to_O_E_converters(CustomPayload& trans, sc_time& delay, int single_wavelength);
    void forward_weight_to_O_E_converters(CustomPayload& trans, sc_time& delay, int cross_wavelength);
    //forward transaction to interposer interface
    void forward_psum_to_interposer(PESendPayload& trans, sc_time& delay);
    
    void log_info(std::string msg);
    void log_error(std::string msg);
};

#endif // CHIPLET_INTERFACE_H