#ifndef GLOBAL_BUFFER_H
#define GLOBAL_BUFFER_H

#include <systemc.h>
#include <tlm.h>
#include <vector>
#include <iostream>
#include <fstream>
#include <sstream>
#include <tlm_utils/simple_initiator_socket.h>
#include <tlm_utils/simple_target_socket.h>
#include "src/utils/configuration.h"
#include "src/utils/noc_logger.h"
#include <cstdlib>  // For rand()
#include <ctime>    // For time()
#include <string>
#include <queue>

using namespace tlm;

class GlobalBuffer : public sc_module {
public:
    // Constructor
    SC_HAS_PROCESS(GlobalBuffer);
    

    // TLM sockets for communication with interposer interfaces   
    tlm_utils::simple_initiator_socket<GlobalBuffer> features_from_GB_sockets[CHIPLET_NO];   // Sending to chiplets
    tlm_utils::simple_initiator_socket<GlobalBuffer> weight_from_GB_sockets[CHIPLET_NO];   // Sending to chiplets
    tlm_utils::simple_target_socket<GlobalBuffer> receive_to_GB_sockets[CHIPLET_NO];  // Receiving PSums from chiplets
    
    explicit GlobalBuffer(sc_module_name name, int num_chiplets, int num_pes);

private:
    // Data members
    
    int num_chiplets;     // Number of chiplets (interposer interfaces)
    int num_pes;          // Number of PEs in total
    std::vector<int> psums; // Storage for received PSums
    std::vector<int> feature_counter;  // Counter for input features
    std::vector<int> weight_counter;   // Counter for weight kernels
    std::vector<std::vector<int>> credit_matrix; // Credit matrix

    // Process declarations
    void generate_features();
    void generate_weights();
    
    // Function to handle received TLM transactions
    void receive_to_gb_b_transport(tlm::tlm_generic_payload& trans, sc_time& delay);
    
    // TLM Communication methods
    void single_chiplet_communication();
    void cross_chiplet_communication();
    //void log_credits(tlm_generic_payload& trans);

    // Function to load multiple matrices from a single file
    std::vector<std::vector<std::vector<int>>> load_matrices_from_file(const std::string& filename);
    bool is_file_empty(const std::string& file_path);
    
    void log_info(std::string msg);
    void log_error(std::string msg);

};

#endif // GLOBAL_BUFFER_H