#ifndef INTERPOSER_INTERFACE_H
#define INTERPOSER_INTERFACE_H

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_target_socket.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <src/utils/configuration.h>
#include <ctime>
#include <src/utils/noc_logger.h>

using namespace tlm;

class InterposerInterface : public sc_module {
public:
    // TLM sockets
    
    tlm_utils::simple_target_socket<InterposerInterface> gb_features_receive_socket; // Receiving from Global Buffer
    tlm_utils::simple_target_socket<InterposerInterface> gb_weight_receive_socket; // Receiving from Global Buffer
    tlm_utils::simple_initiator_socket<InterposerInterface> gb_send_socket;  // Sending to Global Buffer
    tlm_utils::simple_target_socket<InterposerInterface> chiplet_receive_socket; // Receiving from Chiplet Interface
    tlm_utils::simple_initiator_socket<InterposerInterface> chiplet_features_send_socket;  // Sending to Chiplet Interface
    tlm_utils::simple_initiator_socket<InterposerInterface> chiplet_weight_send_socket;  // Sending to Chiplet Interface

    SC_HAS_PROCESS(InterposerInterface);

    InterposerInterface(sc_module_name name);
    int interposer_id;

private:
    // Handle incoming transactions from the Global Buffer
    void gb_features_b_transport(tlm_generic_payload& trans, sc_time& delay);
    void gb_weight_b_transport(tlm_generic_payload& trans, sc_time& delay);
    
    // Handle incoming transactions from the Chiplet Interface
    void chiplet_b_transport(tlm_generic_payload& trans, sc_time& delay);
    
    // Forward transaction to Chiplet Interface
    void chiplet_features_send_transaction(CustomPayload& trans, sc_time& delay, int single_wavelength);
    void chiplet_weight_send_transaction(CustomPayload& trans, sc_time& delay, int cross_wavelength);
    
    // Forward transaction back to Global Buffer
    void gb_send_transaction(PESendPayload& trans, sc_time& delay);
    
    void log_info(std::string msg);
    void log_error(std::string msg);
};

#endif // INTERPOSER_INTERFACE_H