#include "O_E_converter.h"
#include <iostream>

// Constructor implementation
O_E_converter::O_E_converter(sc_module_name name)
    : sc_module(name) {
    //sc_report_handler::set_log_file_name("out/report.log");
    sc_report_handler::set_actions(OEC_LOG, SC_INFO, SC_LOG|SC_DISPLAY);
    // Register b_transport methods for GB and Processing Element s
    ci_features_receive_socket.register_b_transport(this, &O_E_converter::ci_features_b_transport);
    ci_weight_receive_socket.register_b_transport(this, &O_E_converter::ci_weight_b_transport);
    //Register b_transport for incoming psums from PE
    pe_receive_socket.register_b_transport(this, &O_E_converter::pe_b_transport);
    //SC_THREAD(chiplet_send_transaction);
}

// Handle incoming transactions from the chiplet interface
void O_E_converter::ci_features_b_transport(tlm_generic_payload& trans, sc_time& delay) {
    CustomPayload* orig_trans = dynamic_cast<CustomPayload*>(&trans);
    if (!orig_trans) {
        wait(5, SC_NS);
        log_error(std::to_string(O_E_converter_id) + " Invalid transaction type (not CustomPayload)." );
        trans.set_response_status(TLM_GENERIC_ERROR_RESPONSE);
        return;
    }

    CustomPayload* oec_trans1 = new CustomPayload();

    sc_time current_time = sc_time_stamp();
    //manually copy the transaction payload for future use
    // Log the time when the core sent the transaction
    sc_time sent_time = orig_trans->timestamp;
    oec_trans1->data = orig_trans->data;
    oec_trans1->num_rows = orig_trans->num_rows;
    oec_trans1->num_cols = orig_trans->num_cols;
    oec_trans1->update_timestamp(current_time);
    oec_trans1->set_address(orig_trans->get_address());
    oec_trans1->set_command(orig_trans->get_command());
    oec_trans1->set_data_ptr(reinterpret_cast<unsigned char*>(&oec_trans1->data));
    oec_trans1->set_data_length(orig_trans->get_data_length());
    oec_trans1->set_streaming_width(orig_trans->get_streaming_width());
    oec_trans1->set_byte_enable_ptr(orig_trans->get_byte_enable_ptr());
    oec_trans1->set_dmi_allowed(orig_trans->is_dmi_allowed());

    int address = oec_trans1->get_address(); // address is used to determine wavelength
    if (address >= 0x100 && address < 0x200) {
        // Single-chiplet communication
        int chiplet_index = address - 0x100;
        int chiplet_id = chiplet_index - PE_PER_CHIPLET;
        
        int single_wavelength = chiplet_index;
        //log_info(std::to_string(O_E_converter_id) + " Single-chiplet communication, wavelength: " + std::to_string(single_wavelength));
        
        trans.set_response_status(TLM_OK_RESPONSE);  // Set response status
        // Single-chiplet communication (λM)
        log_info(std::to_string(O_E_converter_id) + " Received input feature: " + std::to_string(oec_trans1->data) + " with Wavelength: " + std::to_string(single_wavelength));
        //wait(sc_time(5, SC_NS));
        // Forward the transaction to the Processing Element 
        //sc_time delay = SC_ZERO_TIME;
        delay += sc_time(5, SC_NS);
        pe_features_send_transaction(*oec_trans1, delay, single_wavelength);

    } else {
        log_error(std::to_string(O_E_converter_id) + " Invalid or unrecognized address: " + std::to_string(address));
    }

    delete oec_trans1;  // Clean up dynamically allocated memory    
}

void O_E_converter::pe_features_send_transaction(CustomPayload& trans, sc_time& delay, int single_wavelength){
    log_info(std::to_string(O_E_converter_id) + " Forwarding input features: " + std::to_string(trans.data) + " with Wavelength: " + std::to_string(single_wavelength) + " to Processing Element: " + std::to_string(O_E_converter_id));
    pe_features_send_socket->b_transport(trans, delay);  // Send transaction to Processing Element
    if(trans.get_response_status() == TLM_OK_RESPONSE) {
        log_info(std::to_string(O_E_converter_id) +  " Transaction completed successfully from OEC: " + std::to_string(O_E_converter_id) + " to Processing Element");
    } else {
        log_error( std::to_string(O_E_converter_id) + " Error in transmitting data: " + std::to_string(trans.data) + " from OEC to PE ");
    }
    
}

// Handle incoming weight kernals from the Global Buffer
void O_E_converter::ci_weight_b_transport(tlm_generic_payload& trans, sc_time& delay) {
    CustomPayload* orig_trans = dynamic_cast<CustomPayload*>(&trans);
    if (!orig_trans) {
        
        log_error(std::to_string(O_E_converter_id) + " Invalid transaction type (not CustomPayload).");
        trans.set_response_status(TLM_GENERIC_ERROR_RESPONSE);
        return;
    }

    CustomPayload* oec_trans2 = new CustomPayload();
    sc_time current_time = sc_time_stamp();
    //manually copy the transaction payload for future use
    // Log the time when the core sent the transaction
    sc_time sent_time = orig_trans->timestamp;
    oec_trans2->data = orig_trans->data;
    oec_trans2->num_rows = orig_trans->num_rows;
    oec_trans2->num_cols = orig_trans->num_cols;
    oec_trans2->update_timestamp(current_time);
    oec_trans2->set_address(orig_trans->get_address());
    oec_trans2->set_command(orig_trans->get_command());
    oec_trans2->set_data_ptr(reinterpret_cast<unsigned char*>(&oec_trans2->data));
    oec_trans2->set_data_length(orig_trans->get_data_length());
    oec_trans2->set_streaming_width(orig_trans->get_streaming_width());
    oec_trans2->set_byte_enable_ptr(orig_trans->get_byte_enable_ptr());
    oec_trans2->set_dmi_allowed(orig_trans->is_dmi_allowed());

    int address = oec_trans2->get_address(); // address is used to determine wavelength
    if (address >= 0x200) {
        int cross_wavelength = address - 0x200;
        //log_info("Cross-chiplet communication, wavelength : " + std::to_string(cross_wavelength));
        
        // Cross-chiplet communication (λ0, λ1, λ2, ...)
        log_info(std::to_string(O_E_converter_id) + " Received cross-chiplet data: " + std::to_string(oec_trans2->data) + " with Wavelength: " + std::to_string(cross_wavelength));
        trans.set_response_status(TLM_OK_RESPONSE);    
        // Forward modified data to Processing Element
        //sc_time delay = SC_ZERO_TIME;
        delay += sc_time(5, SC_NS);
        chiplet_weight_send_transaction(*oec_trans2, delay, cross_wavelength);
    } else {
        log_error(std::to_string(O_E_converter_id) + " Invalid or unrecognized address: " + std::to_string(address));    
    }

    delete oec_trans2;  // Clean up dynamically allocated memory   
}

void O_E_converter::chiplet_weight_send_transaction(CustomPayload& trans, sc_time& delay, int cross_wavelength) {

    pe_weight_send_socket->b_transport(trans, delay);  // Send transaction to Processing Element
    log_info(std::to_string(O_E_converter_id) + " Forwarding weight kernals: " + std::to_string(trans.data) + " with Wavelength: " + std::to_string(cross_wavelength) + " to Processing Element: " + std::to_string(cross_wavelength));
    if(trans.get_response_status() == TLM_OK_RESPONSE) {
        log_info( std::to_string(O_E_converter_id) + " Transaction completed successfully from OEC: " + std::to_string(O_E_converter_id) + " to Processing Element");
    } else {
        log_error(std::to_string(O_E_converter_id) +  " Error in transmitting data: " + std::to_string(trans.data) + " from OEC to PE ");
    }
    
}


void O_E_converter::pe_b_transport(tlm_generic_payload& trans, sc_time& delay) {
    PESendPayload* orig_trans = dynamic_cast<PESendPayload*>(&trans);
    if (!orig_trans) {
        
        log_error(std::to_string(O_E_converter_id) + "  Invalid transaction type (not PESendPayload).");
        trans.set_response_status(TLM_GENERIC_ERROR_RESPONSE);
        return;
    }

    PESendPayload* rx_trans1 = new PESendPayload();

    sc_time current_time = sc_time_stamp();
    //manually copy the transaction payload for future use
    // Log the time when the core sent the transaction
    sc_time sent_time = orig_trans->timestamp;
    rx_trans1->psum = orig_trans->psum;
    rx_trans1->src_chiplet = orig_trans->src_chiplet;
    rx_trans1->src_pe = orig_trans->src_pe;
    rx_trans1->num_rows = orig_trans->num_rows;
    rx_trans1->num_cols = orig_trans->num_cols;
    rx_trans1->update_timestamp(current_time);
    rx_trans1->set_address(orig_trans->get_address());
    rx_trans1->set_command(orig_trans->get_command());
    rx_trans1->set_data_ptr(reinterpret_cast<unsigned char*>(&rx_trans1->psum));
    rx_trans1->set_data_length(orig_trans->get_data_length());
    rx_trans1->set_streaming_width(orig_trans->get_streaming_width());
    rx_trans1->set_byte_enable_ptr(orig_trans->get_byte_enable_ptr());
    rx_trans1->set_dmi_allowed(orig_trans->is_dmi_allowed());

    int address = rx_trans1->get_address(); // address is used to determine wavelength

    int chiplet_index = address - 0x300;
   
    int unicast_wavelength = chiplet_index;
    
    log_info(std::to_string(O_E_converter_id) + " Optical to Electrical converter received PSUM from PE: " + std::to_string(rx_trans1->src_pe) + " with data: " + std::to_string(rx_trans1->psum));
    trans.set_response_status(TLM_OK_RESPONSE);  // Set response status

    // Forward the transaction to the Chiplet interface 
    //delay = SC_ZERO_TIME;
    delay += sc_time(5, SC_NS);
    chiplet_send_transaction(*rx_trans1, delay);
    delete rx_trans1;  // Clean up dynamically allocated memory
    
}

void O_E_converter::chiplet_send_transaction(PESendPayload& trans, sc_time& delay){
    ci_send_socket->b_transport(trans, delay);    
    log_info(std::to_string(O_E_converter_id) + " Forwarding the PSUMs from PE: " + std::to_string(trans.src_pe) + " to chiplet interface: " + std::to_string(trans.src_chiplet));
    if(trans.get_response_status() == TLM_OK_RESPONSE) {
        log_info( std::to_string(O_E_converter_id) + " PSUM Transaction completed successfully from OEC: " + std::to_string(O_E_converter_id) + " to Chiplet Interface");
    }else{
        log_error( std::to_string(O_E_converter_id) + " Error in transmitting PSUM: " + std::to_string(trans.psum) + " from OEC to Chiplet Interface ");
    }
    
}

void O_E_converter::log_info(std::string msg){
    SC_REPORT_INFO(OEC_LOG, msg.c_str());
}

void O_E_converter::log_error(std::string msg){
    SC_REPORT_ERROR(OEC_LOG, msg.c_str());
}