#ifndef O_E_CONVERTER_H
#define O_E_CONVERTER_H

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_target_socket.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <src/utils/configuration.h>
#include <src/utils/noc_logger.h>
#include <ctime>

using namespace tlm;

class O_E_converter : public sc_module {
public:
    // TLM sockets
    
    tlm_utils::simple_target_socket<O_E_converter> ci_features_receive_socket; // Receiving from Chiplet Interface
    tlm_utils::simple_target_socket<O_E_converter> ci_weight_receive_socket; // Receiving from Chiplet Interface
    tlm_utils::simple_initiator_socket<O_E_converter> ci_send_socket;  // Sending to Chiplet Interface
    tlm_utils::simple_target_socket<O_E_converter> pe_receive_socket; // Receiving from PE
    tlm_utils::simple_initiator_socket<O_E_converter> pe_features_send_socket;  // Sending to PE
    tlm_utils::simple_initiator_socket<O_E_converter> pe_weight_send_socket;  // Sending to PE

    SC_HAS_PROCESS(O_E_converter);

    O_E_converter(sc_module_name name);

    int O_E_converter_id;

private:
    // Handle incoming transactions from the Chiplet Interface
    void ci_features_b_transport(tlm_generic_payload& trans, sc_time& delay);
    void ci_weight_b_transport(tlm_generic_payload& trans, sc_time& delay);
    
    // Handle incoming transactions from the processing element
    void pe_b_transport(tlm_generic_payload& trans, sc_time& delay);

    //forward transactions to PE
    void pe_features_send_transaction(CustomPayload& trans, sc_time& delay, int single_wavelength);
    void chiplet_weight_send_transaction(CustomPayload& trans, sc_time& delay, int cross_wavelength);
    
    // Forward transaction to Chiplet Interface
    void chiplet_send_transaction(PESendPayload& trans, sc_time& delay);
    

    void log_info(std::string msg);
    void log_error(std::string msg);
};

#endif // INTERPOSER_INTERFACE_H