#pragma once

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <tlm_utils/simple_target_socket.h>
#include <queue>
#include <iostream>
#include <vector>
#include <string>
#include <iostream>
#include <cstdlib>
#include <cstdlib>  // For rand()
#include <ctime>    // For time()

#include "src/utils/configuration.h"
#include "src/utils/token_manager.h"
#include "src/utils/noc_logger.h"

using namespace sc_core;
using namespace tlm;

class ProcessingElement : public sc_module {
public:
    // TLM sockets
    
    tlm_utils::simple_target_socket<ProcessingElement> oec_features_receive_socket; // Receiving from Chiplet Interface
    tlm_utils::simple_target_socket<ProcessingElement> oec_weight_receive_socket; // Receiving from Chiplet Interface
    tlm_utils::simple_initiator_socket<ProcessingElement> oec_send_socket;  // Sending to Chiplet Interface
    
    sc_core::sc_fifo<int> features_fifo;  // FIFO for incoming features transactions from oec
    sc_core::sc_fifo<int> weight_fifo;  // FIFO for incoming weight transactions from oec
    sc_core::sc_fifo<int> psum_fifo;  // FIFO for transactions to be processed by the pe

    sc_core::sc_event psum_written_event;

    SC_HAS_PROCESS(ProcessingElement);

    explicit ProcessingElement(sc_module_name name, TokenManager& token_ref);

    int pe_id;
    int pe_chiplet_id;


    // Add a pointer to the semaphore manager
   // TokenManager* tok_mgr;


private:
    TokenManager& token; // Reference to the Token instance
    // Handle incoming transactions from the OEC
    void oec_features_b_transport(tlm_generic_payload& trans, sc_time& delay);
    void oec_weight_b_transport(tlm_generic_payload& trans, sc_time& delay);

    

    // Thread to process ml
    void process_convolution();

    void process_psum_send();
    
    void log_info(std::string msg);
    void log_error(std::string msg);

    // Method to set the semaphore manager instance after creation
    // void set_token_manager(TokenManager* manager) {
    // tok_mgr = manager;
//  }

};   

