/*******************************************************************************
 * Copyright (C) 2024
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/

#pragma once

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <tlm_utils/simple_target_socket.h>
// #include "ratatoskrUtils/utils/Structures.h"
// #include <ratatoskrUtils/utils/TrafficTracer.h>
// #include <ratatoskrUtils/utils/GlobalResources.h>

using namespace sc_core;  // For sc_time

// Global parameters to generalize the design
#define PE_NO 16 // Number of routers
#define CHIPLET_NO 4 
#define PE_PER_CHIPLET (PE_NO/CHIPLET_NO)  // Number of NIs per router
#define IFMAP_FIFO_CAPACITY 20  // Capacity of FIFO in PE
#define WEIGHT_FIFO_CAPACITY 64 // Capacity of FIFO 
#define ACC_FIFO_CAPACITY 20
#define INTERPOSER_INTERFACE 8 // 
#define NUM_MATRICES 10

#define LOG_NAME "MY_LOG"
#define NI_LOG   "TLM_NI"


struct CustomPayload : public tlm::tlm_generic_payload {
    
  unsigned int num_rows;           // Number of rows in the matrix
  unsigned int num_cols;           // Number of columns in the matrix
  int data;           // Flattened matrix data
  sc_core::sc_time timestamp;      // Timestamp field

  // Default constructor
  CustomPayload()
      : num_rows(0),
        num_cols(0),
        data(0),
        timestamp(sc_core::SC_ZERO_TIME) {
      set_data_ptr(reinterpret_cast<unsigned char*>(&data));
      set_data_length(sizeof(data));
      set_streaming_width(sizeof(data));
  }

  // Constructor with random data
  CustomPayload(unsigned int num_rows, unsigned int num_cols, int data, sc_core::sc_time time = sc_core::SC_ZERO_TIME)
    : num_rows(num_rows),
      num_cols(num_cols),
      data(data),
      timestamp(time) {
    
      set_data_ptr(reinterpret_cast<unsigned char*>(&data));
      set_data_length(sizeof(data));
      set_streaming_width(sizeof(data));    
  }

  // Function to update the timestamp at any stage
  void update_timestamp(sc_core::sc_time new_time) {
      timestamp = new_time;
  }
};

struct PESendPayload : public tlm::tlm_generic_payload {
  unsigned int src_chiplet;
  unsigned int src_pe;
  unsigned int num_rows;           // Number of rows in the matrix
  unsigned int num_cols;           // Number of columns in the matrix
  int psum;           // Flattened matrix data
  sc_core::sc_time timestamp;      // Timestamp field

  // Default constructor
  PESendPayload()
    : src_chiplet(0),
      src_pe(0),
      num_rows(0),
      num_cols(0),
      psum(0),
      timestamp(sc_core::SC_ZERO_TIME) {
    set_data_ptr(reinterpret_cast<unsigned char*>(&psum));
    set_data_length(sizeof(psum));
    set_streaming_width(sizeof(psum));
  }

  // Constructor with random data
  PESendPayload(unsigned int src_chiplet, unsigned int src_pe, unsigned int num_rows, unsigned int num_cols, int psum, sc_core::sc_time time = sc_core::SC_ZERO_TIME)
    : src_chiplet(src_chiplet),
      src_pe(src_pe),
      num_rows(num_rows),
      num_cols(num_cols),
      psum(psum),
      timestamp(time) {
    
      set_data_ptr(reinterpret_cast<unsigned char*>(&psum));
      set_data_length(sizeof(psum));
      set_streaming_width(sizeof(psum));
      
  }

  // Function to update the timestamp at any stage
  void update_timestamp(sc_core::sc_time new_time) {
      timestamp = new_time;
  }
};  
